/***********************************************************************************

    Copyright (C) 2007-2012 Ahmet Öztürk (aoz_2@yahoo.com)

    This file is part of Lifeograph.

    Lifeograph is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Lifeograph is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Lifeograph.  If not, see <http://www.gnu.org/licenses/>.

***********************************************************************************/


#ifndef LIFEOGRAPH_BASE_HEADER
#define LIFEOGRAPH_BASE_HEADER


#include <gtkmm.h>

#include "settings.hpp"
#include "diary.hpp"
#include "panel_main.hpp"
#include "panel_diary.hpp"
#include "panel_extra.hpp"
#include "view_entry.hpp"


namespace LIFEO
{
    static const char   PROGRAM_NAME[]              = "Lifeograph";

    static const int    BACKUP_INTERVAL             = 30;    // seconds
    static const int    LOGOUT_COUNTDOWN            = 10;    // seconds

    static const char   TARGET_ENTRY_DIARY[]        = "GTK_TREE_MODEL_ROW_DIARY";
    static const char   TARGET_ENTRY_TAG[]          = "GTK_TREE_MODEL_ROW_TAG";
    static const char   TARGET_ENTRY_THEME[]        = "GTK_TREE_MODEL_ROW_THEME";
    static const char   TARGET_ENTRY_ENTRY[]        = "GTK_TREE_MODEL_ROW_ENTRY";
    static const char   TARGET_ENTRY_CHAPTER[]      = "GTK_TREE_MODEL_ROW_CHAPTER";

    // GENERIC STRINGS
    namespace STRING
    {
        static const char OFF[] = N_( "Off" );
        // TRANSLATORS: title of an empty diary entry
        static const char EMPTY_ENTRY_TITLE[] = N_( "<empty entry>" );
    }

struct Icons
{
    Glib::RefPtr< Gdk::Pixbuf >     diary_16;
    Glib::RefPtr< Gdk::Pixbuf >     diary_32;
    Glib::RefPtr< Gdk::Pixbuf >     entry_16;
    Glib::RefPtr< Gdk::Pixbuf >     entry_32;
    Glib::RefPtr< Gdk::Pixbuf >     entry_favorite_16;
    Glib::RefPtr< Gdk::Pixbuf >     entry_favorite_32;
    Glib::RefPtr< Gdk::Pixbuf >     tag_16;
    Glib::RefPtr< Gdk::Pixbuf >     tag_32;
    Glib::RefPtr< Gdk::Pixbuf >     tag_category_16;
    Glib::RefPtr< Gdk::Pixbuf >     tag_category_32;
    Glib::RefPtr< Gdk::Pixbuf >     chapter_16;
    Glib::RefPtr< Gdk::Pixbuf >     chapter_32;
    Glib::RefPtr< Gdk::Pixbuf >     theme_16;
    Glib::RefPtr< Gdk::Pixbuf >     theme_default_16;
    Glib::RefPtr< Gdk::Pixbuf >     theme_32;
    Glib::RefPtr< Gdk::Pixbuf >     month_16;
    Glib::RefPtr< Gdk::Pixbuf >     month_32;
    Glib::RefPtr< Gdk::Pixbuf >     filter_16;
};

typedef std::set< std::string > LanguageList;

class AppWindow; // forward declaration

// Lifeograph Application
class Lifeograph : public Gtk::Application
{
    public:
        static const char   PROGRAM_VERSION_STRING[];
//        static const int    PROGRAM_VERSION_INT = 100000;

        enum LoginStatus
        {
            LOGGED_OUT = 0, LOGGED_IN = 1, LOGGED_TIME_OUT = 2
        };


        static Glib::RefPtr< Lifeograph >
                                    create( void );

        static bool                 quit( void );

                                    Lifeograph();
        virtual                     ~Lifeograph();
        static Lifeograph           *p; // static pointer

        void                        on_startup( void );
        void                        on_activate( void );
        int             on_command_line( const Glib::RefPtr< Gio::ApplicationCommandLine >& );

        static Settings             settings;
        static Icons                *icons;

        static Glib::RefPtr< Gtk::Builder >
                                    builder;

        static void                 create_action(
                bool,
                Glib::RefPtr< Gtk::Action >&,
                const Glib::ustring&,
                const Gtk::StockID&,
                const Glib::ustring&,
                const Glib::ustring&,
                const Gtk::AccelKey&,
                const Glib::SignalProxy0< void >::SlotType&,
                Gtk::Activatable* = NULL );

        static void                 create_action(
                bool,
                Glib::RefPtr< Gtk::Action >&,
                const Glib::ustring&,
                const Glib::ustring&,
                const Glib::ustring&,
                const Glib::ustring&,
                const Gtk::AccelKey&,
                const Glib::SignalProxy0< void >::SlotType&,
                Gtk::Activatable* = NULL );

        static void                 create_toggle_action(
                bool,
                Glib::RefPtr< Gtk::ToggleAction >&,
                const Glib::ustring&,
                const Glib::ustring&,
                const Glib::ustring&,
                const Glib::ustring&,
                const Gtk::AccelKey&,
                const Glib::SignalProxy0< void >::SlotType&,
                Gtk::Activatable* = NULL );

        bool                        load_gui( const std::string& );

        // AUTO LOGOUT FUNCTIONALITY
        bool                        handle_idle( void );
        void                        handle_event( void );
        void                        freeze_auto_logout( void );
        void                        unfreeze_auto_logout( void );

        int                         m_seconds_remaining;
        sigc::connection            m_connection_timeout;

        static SignalVoid           s_signal_logout;

        static int                  m_internaloperation;

        // DRAG & DROP
        std::vector< Gtk::TargetEntry >
                                    drag_targets_tag;
        std::vector< Gtk::TargetEntry >
                                    drag_targets_entry;
        static DiaryElement         *s_elem_dragged;
        static bool                 s_flag_dragging;

        // ENCHANT
        static EnchantBroker        *s_enchant_broker;
        static LanguageList         s_lang_list;

        static std::set< std::string > stock_diaries;

        static Glib::RefPtr< Gdk::Pixbuf >
                                    get_thumbnail( const std::string&, int );

        static SignalVoid           signal_logout( void )
        { return s_signal_logout; }

        static Glib::RefPtr< Gtk::ActionGroup >
                                    m_actions_read;
        static Glib::RefPtr< Gtk::ActionGroup >
                                    m_actions_edit;

        static LoginStatus          loginstatus;

        //std::string                 m_program_path;  // XXX NOT USED

        bool                        m_flag_force_welcome;
        bool                        m_flag_open_directly;
        bool                        m_flag_read_only; // for command-line supplied file

        bool                        m_flag_auto_logout_frozen;

    protected:
        static void                 fill_lang_list_cb( const char* const l, const char* const,
                                                       const char* const, const char* const,
                                                       void* )
        { s_lang_list.insert( l ); }

        // APPLICATION MENU
        Glib::RefPtr< Gio::Menu >   m_menu;

        // POOL OF THUMBNAILS FOR IMAGE LINKS IN DIARY
        typedef std::map< std::string, Glib::RefPtr< Gdk::Pixbuf > > MapPixbufs;
        MapPixbufs                  map_thumbnails;
};

} // end of namespace LIFEO

#endif

