/***********************************************************************************

    Copyright (C) 2007-2011 Ahmet Öztürk (aoz_2@yahoo.com)

    This file is part of Lifeograph.

    Lifeograph is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Lifeograph is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Lifeograph.  If not, see <http://www.gnu.org/licenses/>.

***********************************************************************************/


#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "lifeograph.hpp"
#include "app_window.hpp"
#include "view_entry.hpp"
#include "panel_extra.hpp"


using namespace LIFEO;

// PSEUDO DIARYELEMENT CLASS FOR HEADERS ===========================================================
class ListHeader : public DiaryElement
{
    public:
                                    ListHeader( DiaryElement::Type type )
        : DiaryElement( NULL ), m_type( type ) {}

        void                        show( void ) {}

        int                         get_size( void ) const
        { return 0; }   // redundant
        DiaryElement::Type          get_type( void ) const
        { return m_type; }

    protected:
        DiaryElement::Type          m_type;
};

static ListHeader s_header( DiaryElement::ET_HEADER ), s_none( DiaryElement::ET_NONE );

// TAG PANEL =======================================================================================
// STATIC MEMBERS
PanelExtra::Colrec      *PanelExtra::colrec;

PanelExtra::PanelExtra( BaseObjectType *cobject, const Glib::RefPtr<Gtk::Builder>& )
:   Gtk::TreeView( cobject ), m_treestore_all_tags( NULL ), m_flag_themes_expanded( false )
{
    Gtk::CellRendererPixbuf *cellrenderer_icon( NULL );
    Gtk::CellRendererText *cellrenderer_name( NULL );
    Gtk::TreeViewColumn *column_name_all( NULL );

    try
    {
        colrec = new Colrec;

        cellrenderer_icon = Gtk::manage( new Gtk::CellRendererPixbuf );
        cellrenderer_name = Gtk::manage( new Gtk::CellRendererText );
        column_name_all = Gtk::manage( new Gtk::TreeViewColumn( "" ) );
    }
    catch( ... )
    {
        throw LIFEO::Error( "Failed to create the extra panel" );
    }

    cellrenderer_name->property_ellipsize() = Pango::ELLIPSIZE_END;
    //cellrenderer_name->property_scale() = .90;

    column_name_all->pack_start( *cellrenderer_icon, Gtk::PACK_SHRINK );
    column_name_all->pack_start( *cellrenderer_name );
    column_name_all->add_attribute( cellrenderer_icon->property_pixbuf(),
                            PanelExtra::colrec->icon );
    column_name_all->add_attribute( cellrenderer_name->property_markup(),
                            PanelExtra::colrec->name );

    m_treestore_all_tags = Gtk::TreeStore::create( *PanelExtra::colrec );

    // ALL TAGS
    this->set_model( m_treestore_all_tags );
    column_name_all->set_cell_data_func( *cellrenderer_icon,
                                         sigc::mem_fun( this, &PanelExtra::cell_data_func_icon ) );
    column_name_all->set_cell_data_func( *cellrenderer_name,
                                         sigc::mem_fun( this, &PanelExtra::cell_data_func_text ) );
    this->append_column( *column_name_all );
    this->override_background_color(
            AppWindow::p->get_style_context()->get_background_color( Gtk::STATE_FLAG_NORMAL ),
            Gtk::STATE_FLAG_NORMAL );
    this->override_background_color(
            AppWindow::p->get_style_context()->get_background_color( Gtk::STATE_FLAG_SELECTED ),
            Gtk::STATE_FLAG_SELECTED );

    // SIGNALS
    this->signal_row_activated().connect(
            sigc::mem_fun( *this, &PanelExtra::handle_treeview_row_activated ) );

    this->signal_drag_begin().connect_notify(
            sigc::mem_fun( this, &PanelExtra::handle_treeview_drag_begin ) );

    this->signal_drag_data_get().connect_notify(
            sigc::mem_fun( this, &PanelExtra::handle_drag_data_get ) );

    this->signal_row_expanded().connect(
            sigc::mem_fun( this, &PanelExtra::handle_treeview_row_expanded ) );
    this->signal_row_collapsed().connect(
            sigc::mem_fun( this, &PanelExtra::handle_treeview_row_expanded ) );
}

void
PanelExtra::handle_login( void )
{
    if( Diary::d->is_read_only() )
    {
        unset_rows_drag_source();
        unset_rows_drag_dest();
    }
    else
    {
        enable_model_drag_source( Lifeograph::p->drag_targets_tag );
        enable_model_drag_dest( Lifeograph::p->drag_targets_tag, Gdk::ACTION_MOVE );
    }

    populate();
}

void
PanelExtra::handle_logout( void )
{
    m_treestore_all_tags->clear();
}

void
PanelExtra::cell_data_func_icon( Gtk::CellRenderer *cell, const Gtk::TreeModel::iterator &iter )
{
    DiaryElement *elem( ( * iter )[ colrec->ptr ] );
    cell->set_property( "visible", elem->get_type() != DiaryElement::ET_HEADER );
}

void
PanelExtra::cell_data_func_text( Gtk::CellRenderer *cell, const Gtk::TreeModel::iterator &iter )
{
    DiaryElement *elem( ( * iter )[ colrec->ptr ] );

    switch( elem->get_type() )
    {
        case DiaryElement::ET_HEADER:
            //cell->set_property( "background-gdk", Gdk::Color( "#DDDDDD" ) );
            cell->set_property( "scale", 1.0 );
            cell->set_property( "ypad", 5 );
            break;
        default:
            //cell->set_property( "background-set", false );
            cell->set_property( "scale", 0.9 );
            cell->set_property( "ypad", 0 );
            break;
    }
}

void
PanelExtra::populate( void )
{
    Lifeograph::m_internaloperation++;

    m_treestore_all_tags->clear();

    Gtk::TreeRow row;

    // TAGS GROUP
    row = * m_treestore_all_tags->append();
    row[ colrec->ptr ] = &s_header;
    row[ colrec->name ] = Glib::ustring::compose( "<b>%1</b>", _( "TAGS" ) );

    if( Diary::d->get_tags()->size() < 1 && Diary::d->m_tag_categories.size() < 1 )
    {
        row = * m_treestore_all_tags->append();
        row[ colrec->ptr ] = &s_none;
        row[ colrec->name ] = Glib::ustring::compose( "<i>%1</i>", _( "None" ) );
    }

    // ROOT TAGS
    for( PoolTags::const_iterator iter( Diary::d->get_tags()->begin() );
         iter != Diary::d->get_tags()->end();
         ++iter )
    {
        Tag *tag( iter->second );
        if( tag->get_category() == NULL )
            add_elem_to_list( tag );
    }

    // CATEGORIES
    for( PoolCategoriesTags::const_iterator iter_category( Diary::d->m_tag_categories.begin() );
         iter_category != Diary::d->m_tag_categories.end();
         ++iter_category )
    {
        CategoryTags *category = iter_category->second;
        row = * add_elem_to_list( category );

        for( CategoryTags::const_iterator iter_tag = category->begin();
             iter_tag != category->end();
             ++iter_tag )
        {
            add_elem_to_list( *iter_tag, &row.children() );
        }

        if( category->m_flag_expanded )
            this->expand_row( m_treestore_all_tags->get_path( row ), false );
    }

    // EMPTY SEPARATOR ROW
    row = * m_treestore_all_tags->append();
    row[ colrec->ptr ] = &s_none;

    // THEMES
    row = * m_treestore_all_tags->append();
    row[ colrec->ptr ] = &s_header;
    row[ colrec->name ] = Glib::ustring::compose( "<b>%1</b>", _( "THEMES" ) );

    for( PoolThemes::const_iterator iter = Diary::d->get_themes().begin();
         iter != Diary::d->get_themes().end();
         ++iter )
    {
        add_elem_to_list( iter->second/*, &row.children()*/ );
    }

    if( m_flag_themes_expanded )
        this->expand_row( m_treestore_all_tags->get_path( row ), false );

    Lifeograph::m_internaloperation--;
}

inline Gtk::TreeIter
PanelExtra::add_elem_to_list( DiaryElement *elem, const Gtk::TreeNodeChildren *children )
{
    Gtk::TreeRow row = * ( children != NULL ? m_treestore_all_tags->append( *children ) :
                                              m_treestore_all_tags->append() );

    row[ colrec->ptr ] = elem;
    row[ colrec->name ] = elem->get_list_str();
    row[ colrec->icon ] = elem->get_icon();

    elem->m_list_data->treepath = m_treestore_all_tags->get_path( row );

    return row;
}

Gtk::TreeRow
PanelExtra::get_row( const Gtk::TreePath &path )
{
    return( * m_treestore_all_tags->get_iter( path ) );
    // TODO: check validity
}

void
PanelExtra::handle_treeview_drag_begin( const Glib::RefPtr< Gdk::DragContext >& )
{
    Lifeograph::s_flag_dragging = true;
    this->get_selection()->unselect_all();
}

void
PanelExtra::handle_drag_data_get( const Glib::RefPtr<Gdk::DragContext> &context,
                                Gtk::SelectionData &data,
                                guint info, guint time )
{
    if( Lifeograph::s_elem_dragged )
    {
        switch( Lifeograph::s_elem_dragged->get_type() )
        {
            case DiaryElement::ET_TAG:
                data.set( TARGET_ENTRY_TAG, Lifeograph::s_elem_dragged->get_name() );
                break;
            case DiaryElement::ET_THEME:
                data.set( TARGET_ENTRY_THEME, Lifeograph::s_elem_dragged->get_name() );
                break;
            case DiaryElement::ET_TAG_CTG:
                data.set( "N/A", "" );
                break;
            default:
                break;
        }
    }
    else    // should never be the case
    {
        data.set( "N/A", "" );
        PRINT_DEBUG( "Nothing is selected at the drag source" );
    }
}

bool
PanelExtra::on_drag_motion( const Glib::RefPtr< Gdk::DragContext > &context,
                                 int x, int y, guint time )
{
    if( ! Lifeograph::s_flag_dragging )
        return false;

    if( ! Lifeograph::s_elem_dragged )
        return false;

    if( Lifeograph::s_elem_dragged->get_type() != DiaryElement::ET_TAG )
        return false;

    bool retval( false );
    DiaryElement *element;
    Gtk::TreePath path;
    Tag *tag_dragged( dynamic_cast< Tag* >( Lifeograph::s_elem_dragged ) );

    // AUTOSCROLL
    Gdk::Rectangle rect;
    get_visible_rect( rect );
    int new_y = rect.get_y();
    if( y < ( rect.get_height() * 0.15 ) )
    {
        if( new_y > 0 )
        {
            new_y -= 5;
            if( new_y < 0 )
                new_y = 0;
            scroll_to_point( -1, new_y );
        }
    }
    else
    if( y > ( rect.get_height() * 0.85 ) )
    {
        if( get_vadjustment()->get_value() < get_vadjustment()->get_upper() )
        {
            new_y += 5;
            scroll_to_point( -1, new_y );
        }
    }

    // EVALUATE THE HOVERED ELEMENT
    if( get_path_at_pos( x, y, path ) )
    {
        Gtk::TreeRow row( * m_treestore_all_tags->get_iter( path ) );
        element = row[ colrec->ptr ];
        if( element != NULL )
        {
            if( element->get_type() == DiaryElement::ET_TAG_CTG )
            {
                CategoryTags *ctg_dropped( dynamic_cast< CategoryTags* >( element ) );
                if( tag_dragged->get_category() != ctg_dropped )
                    retval = true;
            }
            else if( element->get_type() == DiaryElement::ET_TAG )
            {
                Tag *tag_dropped( dynamic_cast< Tag* >( element ) );
                if( tag_dragged->get_category() != tag_dropped->get_category() )
                    retval = true;
            }
        }
    }

    if( retval )
    {
        m_elem_drop_target = element;
        context->drag_status( Gdk::ACTION_MOVE, time );
        get_selection()->select( path );
    }
    else
        get_selection()->unselect_all();

    return retval;
}

bool
PanelExtra::on_drag_drop( const Glib::RefPtr< Gdk::DragContext > &context,
                               int x, int y, guint time )
{
    bool flag_successful( false );

    if( Lifeograph::s_flag_dragging )
    {
        if( Lifeograph::s_elem_dragged != NULL || m_elem_drop_target != NULL )
        {
            if( Lifeograph::s_elem_dragged->get_type() == DiaryElement::ET_TAG )
            {
                Tag *tag( dynamic_cast< Tag* >( Lifeograph::s_elem_dragged ) );
                if( m_elem_drop_target->get_type() == DiaryElement::ET_TAG_CTG )
                {
                    CategoryTags *ctg( dynamic_cast< CategoryTags* >( m_elem_drop_target ) );
                    tag->set_category( ctg );
                    flag_successful = true;
                }
                else if( m_elem_drop_target->get_type() == DiaryElement::ET_TAG )
                {
                    Tag *target_tag( dynamic_cast< Tag* >( m_elem_drop_target ) );
                    tag->set_category( target_tag->get_category() );
                    flag_successful = true;
                }
            }
        }
    }

    if( flag_successful )
    {
        populate();
    }

    context->drag_finish( flag_successful, false, time );
    return flag_successful;
}

bool
PanelExtra::on_button_press_event( GdkEventButton *event )
{
    Lifeograph::s_flag_dragging = false;
    Lifeograph::s_elem_dragged = NULL;

    Gtk::TreePath path;
    if( this->get_path_at_pos( event->x, event->y, path ) )
    {
        Gtk::TreeRow row = * m_treestore_all_tags->get_iter( path );
        Lifeograph::s_elem_dragged = row[ colrec->ptr ];
        if( Lifeograph::s_elem_dragged == NULL )
            return true;    // ignore
        else
        {
            switch( Lifeograph::s_elem_dragged->get_type() )
            {
                case DiaryElement::ET_TAG:
                case DiaryElement::ET_THEME:
                    break;
                case DiaryElement::ET_TAG_CTG:
                    // TODO: prevent start of a drag action
                    break;
                default:
                    Lifeograph::s_elem_dragged = NULL;
                    return true;
            }
        }
    }
    else
    {
        return true;
    }

    return Gtk::TreeView::on_button_press_event( event );
}

bool
PanelExtra::on_button_release_event( GdkEventButton *event )
{
    if( ! Lifeograph::s_flag_dragging /*&& event->button != 1*/ )
    {
        if( Lifeograph::s_elem_dragged )
            Lifeograph::s_elem_dragged->show();
    }

    get_selection()->unselect_all();

    return Gtk::TreeView::on_button_release_event( event );
}

void
PanelExtra::handle_treeview_row_expanded( const Gtk::TreeIter &iter,
                                        const Gtk::TreePath &path )
{
    if( Lifeograph::m_internaloperation )
        return;

    DiaryElement *element = ( * iter )[ colrec->ptr ];
    if( element )
    {
        if( element->get_type() == DiaryElement::ET_TAG_CTG )
        {
            CategoryTags *category = dynamic_cast< CategoryTags* >( element );
            category->m_flag_expanded = this->row_expanded( path );
        }
    }
    else
        m_flag_themes_expanded = true;  // TODO: in the future there may be other expandable
                                        // items in the list than THEMES so a check is to be added
}

void
PanelExtra::handle_treeview_row_activated( const Gtk::TreePath &path, Gtk::TreeViewColumn* )
{
    Gtk::TreeRow row( * m_treestore_all_tags->get_iter( path ) );
    DiaryElement *element( row[ colrec->ptr ] );

    if( element == NULL )
        return;

    if( element->get_type() == DiaryElement::ET_TAG )
    {
        Tag *tag( dynamic_cast< Tag* >( element ) );

        if( tag == Diary::d->get_filter_tag() ) // remove filter
        {
            Diary::d->set_filter_tag( NULL );
            row[ colrec->icon ] = Lifeograph::icons->tag_16;
        }
        else
        {
            if( Diary::d->get_filter_tag() != NULL )
            {
                Gtk::TreeRow row_old( * m_treestore_all_tags->get_iter(
                        Diary::d->get_filter_tag()->m_list_data->treepath ) );
                row_old[ colrec->icon ] = Lifeograph::icons->tag_16;
            }
            Diary::d->set_filter_tag( tag );
            row[ colrec->icon ] = Lifeograph::icons->filter_16;
        }
        AppWindow::p->m_tag_view->update_button_filter_tag();
        AppWindow::p->panel_diary->update_button_remove_filters();
        AppWindow::p->panel_diary->update_entry_list();
    }
}
